Program CRO;

Uses Graph, Crt;
{$I Maths.CRO}
{$I IO.CRO}

Type
    color = 0..MaxColors;
    paint = 0..63;
    time = double;
    voltage = double;
    deflection = double;
    signal = Function(t:time):voltage;
    deflector = Function(Var t:time):deflection;


Const
     BV = 0.0;
     MaxCh = 5; ChZ = MaxCh+1; ChT = MaxCh+2; Sources = MaxCh+2;
     MaxVar = 10;
     Res = 0.0025;

     BackC = 0;
     BorderC = 8;
     ScreenC = 9;
     GridC = 1;
     PlotC: array[1..MaxCh] of color = (10, 11, 12, 13, 14);
     JoinC: array[1..MaxCh] of color = (2, 3, 4, 5, 6);
     TextC = 15;
     DullC = 7;
     BasePal:palettetype = (Size: 16;
      Colors: (0, 0, 2, 3, 6, 4, 1, 7, 56, 7, 18, 27, 54, 36, 9, 63));

Type
    ChannelData = record
		  Name: string[20];
		  Y: signal;
		  Y0: deflection;
		  VPD: voltage;
		  Gnd: boolean;
		  Disp: boolean;
		  Join: boolean;
		  JoinMask: paint;
		  SY, PSY: deflection;
		  Plot, PPlot: boolean
		  end;
    ExptData = record
	       Name: string[80];
	       SigName: array[1..Sources] of string[20];
	       Signal: array[1..Sources] of signal;
	       Params: 0..MaxVar;
	       Param: array[1..MaxVar] of string[40]
	       end;

Var
   t, dt: time; Ramp: deflection;
   X: deflector;
   X0: deflection;
   TPD: time;
   SX, PSX: double;
   Ch: array[1..MaxCh] of ChannelData;
   Z: signal;
   ZMode: boolean;
   XYMode: boolean;

   Trig, ExtTrig: signal;
   TrigCh: 0..Sources; TLevel: voltage; TSlope: -1..+1;

   Pal: palettetype;
   Scale, CX, CY: integer;

   ExptNo: integer;
   v: array[1..MaxVar] of double;

   C: 1..MaxCh;
   MBtns: byte; K: char;
   i: integer;

{$I Expt-Lib.CRO}
{$I Expt-Ini.CRO}
{$I Internal.CRO}
{$I Screen.CRO}
{$I Panel.CRO}


Label NewExpt, SameExpt, Trace;

Begin

RegisterBGIDriver(@VGADriver);
MBtns := ResetMouse;
InitVars;

NewExpt: GetExpt;
SameExpt: GetParams;

InitScreen;

repeat
      UpdatePanel;
      ClearScreen;
      if XYMode then dt := Res*1e-3 else dt := Res*TPD;

      t := 0-dt;
      Ramp := +6;
      for i := 1 to MaxCh do with Ch[i] do if Disp then
	  Plot := false;

Trace:
      repeat
	    t := t + dt;
	    PSX := SX; SX := X(t) + X0;
	    if not((CtlGrid(t) > BV) and (SX >= -5) and (SX <= 5)) then
	       begin
	       for i := 1 to MaxCh do with Ch[i] do if Disp then
		   Plot := false;
	       continue
	       end;

	    for i:= MaxCh downto 1 do with Ch[i] do if Disp then
		begin
		PSY := SY;
		if Gnd then SY := 0 + Y0 else SY := Y(t)/VPD + Y0;
		PPlot := Plot;
		Plot := (SY >= -4) and (SY <= 4);
		if not(Plot) then continue;
		if Join and (PSX <= SX) and PPlot then
		   begin
		   setcolor(JoinC[i]);
		   line(x_(PSX), y_(PSY), x_(SX), y_(SY));
		   putpixel(x_(PSX), y_(PSY), PlotC[i]);
		   end;
		putpixel(x_(SX), y_(SY), PlotC[i]);
		end;
      until keypressed or CtrlKey;

      if CtrlKey then
	 begin
	 UpdatePanel;
	 while CtrlKey do;
	 continue
	 end;

      K := readkey;
      with Ch[C] do case K of
	   chr(27) : begin closegraph; halt end;
	   chr(13) : begin
		     closegraph;
		     if ShiftKey then
			goto NewExpt
		     else
			goto SameExpt
		     end;
	   chr(8) : begin InitVars; ReInitScreen end;
	   '1'..chr(ord('0')+MaxCh) : Ch[ord(K)-ord('0')].Disp := not(Ch[ord(K)-ord('0')].Disp);
	   ' ' : Gnd := not(Gnd);
	   'Z', 'z' : ZMode := not(ZMode);
	   '*' : begin
		 XYMode := not(XYMode);
		 if XYMode then X := AltX else X := TimeBase
		 end;
	   '[' : TLevel := TLevel - 0.1;
	   ']' : TLevel := TLevel + 0.1;
	   '{' : TLevel := TLevel - 0.01;
	   '}' : TLevel := TLevel + 0.01;
	   '=' : TLevel := 0;
	   '/' : TSlope := +1;
	   '\' : TSlope := -1;
	   chr(9) : Join := not(Join);
	   '+' : if (Scale < 110) then SetScale(Scale+5) else goto Trace;
	   '-' : if (Scale > 5) then SetScale(Scale-5) else goto Trace;
	   'S', 's' : begin if ShiftKey then DecPal(ScreenC) else IncPal(ScreenC); goto Trace end;
	   'G', 'g' : begin if ShiftKey then DecPal(GridC) else IncPal(GridC); goto Trace end;
	   'T', 't' : begin
		      if ShiftKey then DecPal(PlotC[C]) else IncPal(PlotC[C]);
		      SetPal(JoinC[C], Pal.Colors[PlotC[C]] and JoinMask);
		      goto Trace
		      end;
	   'M', 'm' : if MBtns > 0 then
			 Measure
		      else
			 begin
			 Beep;
			 goto Trace
			 end;
      chr(0) : begin K := readkey; case K of
	   'I' : if C > 1 then
		    begin
		    Dec(C);
		    if Scale < 50 then begin UpdatePanel; goto Trace end
		    end
		 else
		    goto Trace;
	   'Q' : if C < MaxCh then
		    begin
		    Inc(C);
		    if Scale < 50 then begin UpdatePanel; goto Trace end
		    end
		 else
		    goto Trace;
	   chr(59)..chr(59+MaxCh-1) : begin
				      C := ord(K) - 59 + 1;
				      if Scale < 50 then begin UpdatePanel; goto Trace end
				      end;
	   'R' : Disp := true;
	   'S' : Disp := false;
	   'K' : if ShiiftKey then
		    X0 := X0 - 0.02
		 else if ScrollKey then
		    X0 := X0 - 0.2
		 else
		    if XYMode then Contract(Ch[2].VPD) else Contract(TPD);
	   'M' : if ShiiftKey then
		    X0 := X0 + 0.02
		 else if ScrollKey then
		    X0 := X0 + 0.2
		 else
		    if XYMode then Expand(Ch[2].VPD) else Expand(TPD);
	   'H' : if ShiiftKey then
		    Y0 := Y0 + 0.02
		 else if ScrollKey then
		    Y0 := Y0 + 0.2
		 else
		    Expand(VPD);
	   'P' : if ShiiftKey then
		    Y0 := Y0 - 0.02
		 else if ScrollKey then
		    Y0 := Y0 - 0.2
		 else
		    Contract(VPD);
	   chr(129) : TrigCh := 0;
	   chr(120)..chr(120+MaxCh-1) : begin
					TrigCh := ord(K) - 120 + 1;
					Trig := Ch[TrigCh].Y
					end;
	   chr(18) : begin
		     TrigCh := ChT;
		     Trig := ExtTrig
		     end;
	   chr(15) : begin
		     JoinMask := JoinMask xor 56;
		     SetPal(JoinC[C], Pal.Colors[PlotC[C]] and JoinMask)
		     end;
	   'G' : begin LoadVars; ReInitScreen end;
	   'O' : begin SaveVars; goto Trace end;
	   else goto Trace
      end end;
	   '?' : begin
		 Help;
		 if readkey = chr(0) then readkey;
		 cleardevice
		 end;
	   else goto Trace
      end
until false;

End.